<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Modules\ContaPagar\Factory\ControleTituloAlteradoFactory;

class ControleTituloAlteradoService {

    const BAIXADA = 'B';
    const TITULO_NAO_ENVIADO = '00';
    const RETORNO_COM_ERRO = '01';

    public function __construct(EntityManager $entityManager, Environment $environment, ControleTituloAlteradoFactory $controleTituloAlteradoFactory)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->controleTituloAlteradoFactory = $controleTituloAlteradoFactory;
    }

    public function validaContaAlterada($cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $dtatuavenpag, $vratuapag)
    {
        try {
            $nrorg = $this->environment->getNrOrgTrab();
            $contaPagar = $this->controleTituloAlteradoFactory->getContaPagar($nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag);
            $boletoPagar = $this->controleTituloAlteradoFactory->getBoletPagar($nrorg, $contaPagar->getCdfilibpag(), $contaPagar->getNrlanbolpag());
            $this->existeTituloPagar($cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag);
            $this->validaTituloBaixado($contaPagar);
            $this->validaTituloProrrogado($contaPagar, $dtatuavenpag);
            $this->validaConsistenciaValores($contaPagar, $vratuapag);
            $this->validaSituacaoBoleto($boletoPagar);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function existeTituloPagar($cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag){
        $contaPagar = $this->controleTituloAlteradoFactory->existeTituloPagar($cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag);
        if ($contaPagar[0]['NRCNT'] == 0){
            if ($contaPagar[0]['NRDOC'] == 0){
                throw new \Exception('Documento foi excluído por outro usuário.');
            }else{
                throw new \Exception('Operação bloqueada! Conta foi parcelada.');
            }
        }
    }

    public function validaTituloBaixado($contaPagar){
        if ($contaPagar->getIdstatpag() == self::BAIXADA){
            throw new \Exception('Operação bloqueada! Conta foi baixada.');
        }
    }

    public function validaTituloProrrogado($contaPagar, $dtatuavenpag){
        if (!DateUtil::comparaDatas($contaPagar->getDtatuavenpag(), $dtatuavenpag, '3')){
            throw new \Exception('Operação bloqueada! Conta foi prorrogada.');
        }
    }

    public function validaConsistenciaValores($contaPagar, $vratuapag){
        if (floatval($contaPagar->getVratuapag()) != floatval($vratuapag)){
            throw new \Exception('Operação bloqueada! Valor da conta foi alterado.');
        }
    }

    public function validaSituacaoBoleto($boletoPagar){
        if ($boletoPagar->getIdsitucnabp()){
            if ($boletoPagar->getIdsitucnabp() != self::TITULO_NAO_ENVIADO && $boletoPagar->getIdsitucnabp() != self::RETORNO_COM_ERRO){
                throw new \Exception('Conta de um boleto enviado ao banco (via CNAB) não pode ser alterada.');
            }
        }
        if ($boletoPagar->getNrbaixapag()){
            throw new \Exception('Operação bloqueada! Conta foi baixada.');
        }
    }
}